--------------------------------------------------------------------------------------
-- Cab signal pulse transmitter. Cancels all cab signaling.
-- DRB from KUJU/RSDL
--------------------------------------------------------------------------------------


-- Kuju specific definitions.

-- States
CLEAR = 0
WARNING = 1
BLOCKED = 2

-- Signal Messages (0-9 are reserved by code)
RESET_SIGNAL_STATE						= 0
INITIALISE_SIGNAL_TO_BLOCKED 			= 1
JUNCTION_STATE_CHANGE					= 2
INITIALISE_TO_PREPARED					= 3

-- Locally defined signal mesages
OCCUPATION_INCREMENT					= 10
OCCUPATION_DECREMENT					= 11

SIGNAL_BLOCKED							= 12	-- Stop
SIGNAL_CLEARED							= 13	-- Clear
SIGNAL_WARNING							= 14	-- Approach
SIGNAL_WARNING2							= 15	-- Approach at next signal

-- GERMAN SPECIFIC functionality: custom signal message used to set lights green or reset to red after consist pass
SIGNAL_PREPARE_FOR_TRAIN				= 16
SIGNAL_RESET_AFTER_TRAIN_PASS			= 17

-- US SPECIFIC functionality: signals need to know what sort of train is approaching them
SIGNAL_PREPARE_FOR_PASSENGER			= 16
SIGNAL_PREPARE_FOR_FREIGHT				= 17

-- special signal messages only used by the signals that message the opposite facing signal
-- in the section of track where two single direction tracks converge to one dual direction track
	-- currently only used for UK Semaphore signals
OCCUPATION_REVERSE_INCREMENT  			= 18
OCCUPATION_REVERSE_DECREMENT 			= 19

-- US SPECIFIC functionality: signals need to know speed limit for junction they span (if any) and next signal up line
-- NOTE: RESTRICTED may not apply to all diverging links of a signal, so they're signal rather switch speed states
SIGNAL_RESTRICTED						= 20	-- Restricting - 15mph
SIGNAL_APPROACH_RESTRICTED				= 21	-- Approach Restricting
SIGNAL_APPROACH_RESTRICTED2				= 22	-- Approach Restricting at next signal
SPEED_SLOW								= 23	-- 40mph
SPEED_MEDIUM							= 24	-- 50mph
SPEED_LIMITED							= 25	-- 60mph

-- US SPECIFIC functionality: passed back to let the previous signal know whether the next junction will be diverging or not
SIGNAL_DIVERGING						= 26
SIGNAL_STRAIGHT							= 27

-- Signal Messages 28-39 are available for later extension or end-users

-- What you need to add to a signal message number to turn it into the equivalent PASS_ message
PASS_OFFSET								= 40

-- Pass on messages to handle overlapping links (eg for converging junctions / crossovers)
PASS_RESET_SIGNAL_STATE					= PASS_OFFSET + RESET_SIGNAL_STATE				-- Never used!
PASS_INITIALISE_SIGNAL_TO_BLOCKED		= PASS_OFFSET + INITIALISE_SIGNAL_TO_BLOCKED
PASS_JUNCTION_STATE_CHANGE				= PASS_OFFSET + JUNCTION_STATE_CHANGE
PASS_INITIALISE_TO_PREPARED				= PASS_OFFSET + INITIALISE_TO_PREPARED
PASS_OCCUPATION_INCREMENT				= PASS_OFFSET + OCCUPATION_INCREMENT
PASS_OCCUPATION_DECREMENT				= PASS_OFFSET + OCCUPATION_DECREMENT
PASS_SIGNAL_BLOCKED						= PASS_OFFSET + SIGNAL_BLOCKED
PASS_SIGNAL_CLEARED						= PASS_OFFSET + SIGNAL_CLEARED
PASS_SIGNAL_WARNING						= PASS_OFFSET + SIGNAL_WARNING
PASS_SIGNAL_WARNING2					= PASS_OFFSET + SIGNAL_WARNING2
PASS_SIGNAL_PREPARE_FOR_TRAIN			= PASS_OFFSET + SIGNAL_PREPARE_FOR_TRAIN
PASS_SIGNAL_RESET_AFTER_TRAIN_PASS 		= PASS_OFFSET + SIGNAL_RESET_AFTER_TRAIN_PASS
PASS_SIGNAL_PREPARE_FOR_PASSENGER 		= PASS_OFFSET + SIGNAL_PREPARE_FOR_PASSENGER
PASS_SIGNAL_PREPARE_FOR_FREIGHT 		= PASS_OFFSET + SIGNAL_PREPARE_FOR_FREIGHT
PASS_OCCUPATION_REVERSE_INCREMENT		= PASS_OFFSET + OCCUPATION_REVERSE_INCREMENT
PASS_OCCUPATION_REVERSE_DECREMENT		= PASS_OFFSET + OCCUPATION_REVERSE_DECREMENT
PASS_SIGNAL_RESTRICTED					= PASS_OFFSET + SIGNAL_RESTRICTED
PASS_SIGNAL_APPROACH_RESTRICTED			= PASS_OFFSET + SIGNAL_APPROACH_RESTRICTED
PASS_SIGNAL_APPROACH_RESTRICTED2		= PASS_OFFSET + SIGNAL_APPROACH_RESTRICTED2
PASS_SPEED_SLOW							= PASS_OFFSET + SPEED_SLOW
PASS_SPEED_MEDIUM						= PASS_OFFSET + SPEED_MEDIUM
PASS_SPEED_LIMITED						= PASS_OFFSET + SPEED_LIMITED
PASS_SIGNAL_DIVERGING					= PASS_OFFSET + SIGNAL_DIVERGING
PASS_SIGNAL_STRAIGHT					= PASS_OFFSET + SIGNAL_STRAIGHT


-- US specific: Consist Type constants to determine type of train
CONSIST_TYPE_SPECIAL					= 0
CONSIST_TYPE_LIGHT_ENGINE				= 1
CONSIST_TYPE_EXPRESS_PASSENGER			= 2
CONSIST_TYPE_STOPPING_PASSENGER			= 3
CONSIST_TYPE_HIGH_SPEED_FREIGHT			= 4
CONSIST_TYPE_EXPRESS_FREIGHT			= 5
CONSIST_TYPE_STANDARD_FREIGHT			= 6
CONSIST_TYPE_LOW_SPEED_FREIGHT			= 7
CONSIST_TYPE_OTHER_FREIGHT				= 8
CONSIST_TYPE_EMPTY_STOCK				= 9
CONSIST_TYPE_INTERNATIONAL				= 10

-- Signals ultimately only care if it's freight or passenger though...
CONSIST_TYPE_FREIGHT					= SIGNAL_PREPARE_FOR_FREIGHT
CONSIST_TYPE_PASSENGER					= SIGNAL_PREPARE_FOR_PASSENGER

-- SPAD message to pass to consist
AWS_MESSAGE								= 11
TPWS_MESSAGE							= 12
SPAD_MESSAGE 							= 14

-- End of Kuju specific definitions.

-- Kuju custom signal message missing from default definitions!
	SIGMSG_CUSTOM = 15

-- Custom signal states used for cab signal scripting.
	SIG_NONE = 0
	SIG_CLEAR = 1
	SIG_APPMED = 2
	SIG_APP = 3
	SIG_REST = 4

-- Message directions.
	ADV = 1
	REAR = -1

--------------------------------------------------------------------------------------
-- INITIALISE
--
function Initialise ()

-- Nothing to do.

end

--------------------------------------------------------------------------------------
-- ON CONSIST PASS
--
-- Called by core code when a train passes one of the signal's links.
-- Is continuously called for 100m either side of a signal link.
-- Only deals with trains travelling in the correct direction!

function OnConsistPass ( prevFrontDist, prevBackDist, frontDist, backDist, linkIndex )

-- if the consist is passing in the normal direction and train is over the link.
	if ( frontDist < prevFrontDist ) and ( frontDist < 0 ) and ( backDist > 0 ) then

-- Send signal reset to consist.
		Call( "SendConsistMessage", SIGMSG_CUSTOM, "sig0" )
	end
end

--------------------------------------------------------------------------------------
-- ON SIGNAL MESSAGE
--
function OnSignalMessage( message, parameter, direction, linkIndex )

	save_message = message

-- No processing to do.

-- Forward all messages from whatever link received them.

	if ( parameter ~= "DoNotForward" ) then
		Call( "SendSignalMessage", save_message, parameter, -direction, 1, linkIndex )
	end
end




