------------------------------------------------------------
-- Simulation file for the F40PH
------------------------------------------------------------
-- Dave B
--

-- Constants

-- PCS will trip above this speed.
	MAXLOCOSPD = 103
-- Speeds for upwards transistions.
	TRANSSPD1 = 35
	TRANSSPD2 = 65
-- Cab signals.
	NONE = 0
	CLEAR = 1
	APPMED = 2
	APP = 3
	REST = 4
-- Cab signal speed limits.
	APPMEDSPD = 45
	APPSPD = 30
	RESTSPD = 15

------------------------------------------------------------
-- Setup
------------------------------------------------------------
-- Called when the engine script is initialised
------------------------------------------------------------

function Setup ()

	gPCStime = 0
	gLocoBrkSet = 0
	gTrans = 0
	gLastBrakeHdl = 0
	gLastReverseHdl = 0
	gCurSpdLimit = MAXLOCOSPD
	gLastCabSig = NONE

	gBlendedOn = 0

	gDampedBogie = {}
	for i=0, 9 do
		gDampedBogie[i] = 0
	end
	gBogieindex = 0
	gDampedAppl = {}
	for i=0, 39 do
		gDampedAppl[i] = 0
	end
	gApplindex = 0
	gDampedSupr = {}
	for i=0, 19 do
		gDampedSupr[i] = 0
	end
	gSuprindex = 0
	gDampedAmp = {}
	for i=0, 9 do
		gDampedAmp[i] = 0
	end
	gAmpindex = 0

end

------------------------------------------------------------
-- Update
------------------------------------------------------------
-- Called every frame to update the simulation
------------------------------------------------------------
-- Parameters:
--	inteval = time since last update
------------------------------------------------------------

function Update (inteval)

-- Get engine and core sim parameters.
	CurrentTime = Call( "*:GetSimulationTime", 0 )
	CurrentSpeed = Call( "*:GetControlValue", "SpeedometerMPH", 0 )
	BrakeHdl = Call( "*:GetControlValue", "trainbrkhdl", 0 )
	LocoHdl = Call( "*:GetControlValue", "locobrkhdl", 0 )
	DynoHdl = Call( "*:GetControlValue", "dynohdl", 0 )
	Amps = Call( "*:GetControlValue", "Ammeter", 0 )
	TrainBrake = Call( "*:GetControlValue", "TrainBrakeControl", 0 )
	LocoPressure = Call( "*:GetControlValue", "LocoBrakeCylinderPressurePSI", 0 )
	ResPressure = Call( "*:GetControlValue", "MainReservoirPressurePSI", 0 )
	BrakePipe = Call( "*:GetControlValue", "AirBrakePipePressurePSI", 0 )
	ThrottleHdl = Call( "*:GetControlValue", "throttlehdl", 0 )
	ReverseHdl = Call( "*:GetControlValue", "reversehdl", 0 )
	GenField = Call( "*:GetControlValue", "genfieldbut", 0 )
	FuelPump = Call( "*:GetControlValue", "fuelpumpbut", 0 )
	EngCntrl = Call( "*:GetControlValue", "enginecntrlbut", 0 )
	PCSstate = Call( "*:GetControlValue", "PCSopen", 0 )
	Cutoff = Call( "*:GetControlValue", "cutoff", 0 )
	EmergSet = Call( "*:GetControlValue", "EmergencyBrake", 0 )
	SpeedCntrl = Call( "*:GetControlValue", "speedcntrl", 0 )
	CabSignal = Call( "*:GetControlValue", "cabsignal", 0 )
	ActRevs = Call( "*:GetControlValue", "RPM", 0 )
	HEPRevs = Call( "*:GetControlValue", "RPM_HEP", 0 )
	AlertReset = Call( "*:GetControlValue", "AlertReset", 0 )
	AlertWarn = Call( "*:GetControlValue", "AlertWarn", 0 )


-- 26L Train brake. Includes dynamic brake blending.

	Handleoff = 0
	if ( gLastBrakeHdl == BrakeHdl ) then

		if ( BrakeHdl == 0 ) then
			BrakeSet = 0
			gBlendedOn = 0
		elseif ( BrakeHdl > 0 ) and ( BrakeHdl < 0.14 ) then
			BrakeHdl = 0.1
			BrakeSet = BrakeHdl
			gBlendedOn = 1
		elseif ( BrakeHdl >= 0.14 ) and ( BrakeHdl <= 0.55 ) then
			BrakeSet = BrakeHdl
			gBlendedOn = 1
		elseif ( BrakeHdl > 0.55 ) and ( BrakeHdl < 0.75 ) then
			BrakeHdl = 0.62
			BrakeSet = BrakeHdl
			gBlendedOn = 1
		elseif ( BrakeHdl >= 0.75 ) and ( BrakeHdl < 1 ) then
			Handleoff = 1
			gBlendedOn = 0
		else
			BrakeSet = 1
			gBlendedOn = 0
			Call( "*:SetControlValue", "TrainBrakeControl", 0, 1 )
		end

		if ( gBlendedOn == 1 ) and ( ThrottleHdl == 0 ) and ( Cutoff ~= 0 ) then
			BlendedBrake = TrainBrake / 0.54
		else
			BlendedBrake = 0
		end

		if ( EmergSet == 0 ) and ( Handleoff == 0 ) and ( PCSstate ~= 3 ) then
			if ( Cutoff == 2 ) or (( Cutoff == 1 ) and (( BrakeSet > TrainBrake ) or ( BrakeSet == 0 ))) then
				Call( "*:SetControlValue", "TrainBrakeControl", 0, BrakeSet )
			end
		end

	end

	gLastBrakeHdl = BrakeHdl
	Call( "*:SetControlValue", "trainbrkhdl", 0, BrakeHdl )

-- 26L Loco brake.

	Call( "*:SetControlValue", "EngineBrakeControl", 0, LocoHdl )

-- Loco brake cylinder pressure display. Bail when dynamic brake is operating. Leave 5PSI for rigging.

	if ( Amps > -50 ) or ( LocoHdl > 0 ) then
		gDampedBogie[gBogieindex] = LocoPressure
	else
		gDampedBogie[gBogieindex] = 5
	end

	gBogieindex = gBogieindex + 1
	if (gBogieindex == 10) then
		gBogieindex = 0
	end

	BogiePress = 0
	for i=0, 9 do
		BogiePress = BogiePress + gDampedBogie[i]
	end
	BogiePress = BogiePress / 10

	Call( "*:SetControlValue", "locobrakeneedle", 0, BogiePress )

-- Supression pipe display. 64PSI with brake handle in supression, else 0PSI.

	if ( ( BrakeHdl > 0.61 ) and ( BrakeHdl < 0.63 ) ) then
			gDampedSupr[gSuprindex] = 64
	else
			gDampedSupr[gSuprindex] = 0
	end

	gSuprindex = gSuprindex + 1
	if (gSuprindex == 20) then
		gSuprindex = 0
	end

	SuprPress = 0
	for i=0, 19 do
		SuprPress = SuprPress + gDampedSupr[i]
	end
	SuprPress = SuprPress / 20

	Call( "*:SetControlValue", "SuppressionPipe", 0, SuprPress )

-- Application pipe display. Main res pressure if no penalty, 0 PSI with penalty and no suppression, 64PSI with penalty and suppression.

	if ( PCSstate >= 2 ) and ( CurrentTime > gPCStime + 8 ) then
		if ( ( BrakeHdl > 0.61 ) and ( BrakeHdl < 0.63 ) ) then
			gDampedAppl[gApplindex] = 64
		else
			gDampedAppl[gApplindex] = 0
		end
	else
			gDampedAppl[gApplindex] = ResPressure
	end

	gApplindex = gApplindex + 1
	if (gApplindex == 40) then
		gApplindex = 0
	end

	ApplPress = 0
	for i=0, 39 do
		ApplPress = ApplPress + gDampedAppl[i]
	end
	ApplPress = ApplPress / 40

	Call( "*:SetControlValue", "ApplicationPipe", 0, ApplPress )


-- Power and reverser. Including simulation of transistions/field shunts.

	if ( DynoHdl > 0.1 ) then
		Call( "*:SetControlValue", "throttlehdl", 0, 0 )
	elseif ( EngCntrl == 1 ) and ( PCSstate < 3 ) then
		if ( ThrottleHdl == 0 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0 )
			gTrans = 0
		elseif ( CurrentSpeed > TRANSSPD1 ) and ( gTrans == 0 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gTrans = 1
		elseif (( CurrentSpeed + 5 ) < TRANSSPD1 ) and ( gTrans == 1 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gTrans = 0
		elseif ( CurrentSpeed > TRANSSPD2 ) and ( gTrans == 1 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gTrans = 2
		elseif (( CurrentSpeed + 5 ) < TRANSSPD2 ) and ( gTrans == 2 ) then
			Call( "*:SetControlValue", "Regulator", 0, 0.01 )
			gTrans = 1
		else
			Call( "*:SetControlValue", "Regulator", 0, ThrottleHdl )
		end
	else
		Call( "*:SetControlValue", "Regulator", 0, 0 )
	end

	if ( ThrottleHdl > 0.1 ) or ( DynoHdl > 0.1 ) then
		ReverseHdl = gLastReverseHdl
		Call( "*:SetControlValue", "reversehdl", 0, ReverseHdl )
	end
	gLastReverseHdl = ReverseHdl

	if ( GenField == 1 ) and ( PCSstate < 3 ) then
		Call( "*:SetControlValue", "Reverser", 0, ReverseHdl )
	else
		Call( "*:SetControlValue", "Reverser", 0, 0 )
	end

-- Dynamic brake, included blended braking.

	DynoSet = DynoHdl
	if ( ReverseHdl == 0 ) or ( ThrottleHdl > 0.1 ) then
		DynoHdl = 0
		DynoSet = 0
	elseif ( gLastDynoHdl == DynoHdl ) then
		if ( DynoHdl == 0 ) then
			DynoSet = 0
		elseif ( DynoHdl > 0 ) and ( DynoHdl < 0.15 ) then
			DynoHdl = 0.137
			DynoSet = DynoHdl
		else
			DynoSet = DynoHdl
		end
		if ( DynoSet > BlendedBrake ) and ( CurrentSpeed > 1 ) then
			Call( "*:SetControlValue", "DynamicBrake", 0, DynoSet )
		end
	end

	if ( BlendedBrake > DynoSet ) and ( ReverseHdl ~= 0 ) and ( CurrentSpeed > 1 ) then
		Call( "*:SetControlValue", "DynamicBrake", 0, BlendedBrake )
	end

	if (( BlendedBrake == 0 ) and ( DynoSet == 0 )) or ( ReverseHdl == 0 ) or ( CurrentSpeed <= 1 ) then
		Call( "*:SetControlValue", "DynamicBrake", 0, 0 )
	end

	gLastDynoHdl = DynoHdl
	Call( "*:SetControlValue", "dynohdl", 0, DynoHdl )

-- Animate dynamic brake fan.
	if (( DynoSet ~= 0 ) or ( BlendedBrake ~= 0 )) and ( CurrentSpeed > 1 ) then
		if ( Call( "*:AddTime", "fand", inteval) ~= 0 ) then
			Call( "*:Reset", "fand" )
		end
	end


-- Damped ammeter with field shunts.

	if ( Amps < 0 ) or ( gTrans == 0 ) then
		gDampedAmp[gAmpindex] = Amps
	elseif ( gTrans == 1 ) then
		gDampedAmp[gAmpindex] = Amps*2
	else
		gDampedAmp[gAmpindex] = Amps*4
	end

	gAmpindex = gAmpindex + 1
	if (gAmpindex == 10) then
		gAmpindex = 0
	end

	AmpVal = 0
	for i=0, 9 do
		AmpVal = AmpVal + gDampedAmp[i]
	end
	AmpVal = AmpVal / 10

	Call( "*:SetControlValue", "AmpGauge", 0, AmpVal )


-- Pneumatic control switch. Will be suppressed by brake in suppression position, or speed control off.

	if ( SpeedCntrl == 1 ) then

		if ( CurrentSpeed > gCurSpdLimit ) then
			if ( gPCStime == 0 ) then
				Call( "*:SetControlValue", "PCSopen", 0, 2 )
				gPCStime = CurrentTime
			elseif ( CurrentTime > gPCStime + 8 ) and (( BrakeHdl < 0.61 ) or ( BrakeHdl > 0.63 )) then
				Call( "*:SetControlValue", "PCSopen", 0, 3 )
				Call( "*:SetControlValue", "TrainBrakeControl", 0, 1 )
			end
		elseif ( CurrentSpeed <= gCurSpdLimit ) and ( PCSstate < 3 ) then
			Call( "*:SetControlValue", "PCSopen", 0, 0 )
			gPCStime = 0
		end

	end

	if ( EmergSet == 1 ) or (( BrakeHdl == 1 ) and ( ReverseHdl ~= 0 )) then
		Call( "*:SetControlValue", "PCSopen", 0, 3 )
	end

	if ( EmergSet == 0 ) and ( ThrottleHdl == 0 ) and ( BrakeHdl > 0.61 ) and ( BrakeHdl < 0.63 ) then
		if ( CurrentSpeed > gCurSpdLimit ) and ( SpeedCntrl == 1 ) then
			Call( "*:SetControlValue", "PCSopen", 0, 2 )
		else
			Call( "*:SetControlValue", "PCSopen", 0, 0 )
		end
	end

-- Cab signal based speed limit setting.
-- Cab signal state is set in engine script.

	if ( CabSignal == REST ) then
		gCurSpdLimit = RESTSPD
	elseif ( CabSignal == APP ) then
		gCurSpdLimit = APPSPD
	elseif ( CabSignal == APPMED ) then
		gCurSpdLimit = APPMEDSPD
	else
		gCurSpdLimit = MAXLOCOSPD
	end

-- Alerter for cab signal change.

	if AlertReset > 0.5 then
		Call( "*:SetControlValue", "AlertWarn", 0, 0 )
		AlertWarn = 0
		gLastCabSig = CabSignal
	end

-- Warn for worse aspect change.
	if CabSignal ~= gLastCabSig then
		if CabSignal > gLastCabSig then
			Call( "*:SetControlValue", "AlertWarn", 0, 1 )
			AlertWarn = 1
		end
		if AlertWarn == 0 then
			gLastCabSig = CabSignal
		end
	end

-- HEP based RPM sound control faking.

	if ( Cutoff == 2) and ( EngCntrl == 1 ) then
		if HEPRevs < 900 then
			Call( "*:SetControlValue", "RPM_HEP", 0, HEPRevs + 1 )
		end
	else
		if HEPRevs < ActRevs then
			Call( "*:SetControlValue", "RPM_HEP", 0, HEPRevs + 1 )
		elseif HEPRevs > ActRevs then
			Call( "*:SetControlValue", "RPM_HEP", 0, HEPRevs - 1 )
		end
	end


end

